from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any

from dj_settings import ConfigParser

from pvenv.subcommands.base import BaseCommand

if TYPE_CHECKING:
    from argparse import Namespace
    from pathlib import Path


class Command(BaseCommand):
    def __init__(self, options: Namespace) -> None:
        super().__init__(options)
        self.base_dir: Path = options.base_dir
        self.venv: str = options.venv
        self.cd: bool = options.cd

    def run(self) -> None:
        venv_path = self.base_dir.joinpath(self.venv)
        if not venv_path.exists():
            msg = f"Venv {self.venv} doesn't exist, aborting..."
            raise RuntimeError(msg)

        if os.getenv("VIRTUAL_ENV"):
            self.print("dvenv")

        project = venv_path.joinpath(".project")
        if project.exists():
            if self.cd:
                with project.open() as file:
                    self.print(f"cd {file.read().strip()}")
            environment = venv_path.joinpath(".environment")
            if environment.exists():
                new_environment: dict[str, Any] = {}
                with environment.open() as file:
                    for line in file:
                        new_environment.update(ConfigParser([line.strip()]).data)
                environment_string = " ".join(
                    f"{key}={value}" for key, value in new_environment.items()
                )
                self.print(f"invenv {environment_string}")

        activate = venv_path.joinpath("bin/activate")
        if activate.exists():
            self.print(f". {activate}")
        else:
            self.print(f"export VIRTUAL_ENV=dummy_{self.venv}")



import os
import sys
from argparse import ArgumentParser, Namespace
from pathlib import Path

from pvenv.__version__ import __version__

sys.tracebacklimit = 0


def get_default_base() -> Path:
    default = "~/.local/share/virtualenvs"
    return Path(os.getenv("PVENV_BASE", default)).expanduser().absolute()


def parse_args() -> Namespace:
    parser = ArgumentParser(
        prog="pvenv",
        description="A utility to manage virtual environments",
        epilog="You can find the full documentation at: https://p-venv.readthedocs.io/en/stable/",
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"%(prog)s {__version__}",
        help="print the version and exit",
    )

    parent_parser = ArgumentParser(add_help=False)
    parent_parser.add_argument(
        "base_dir", action="store_const", const=get_default_base()
    )
    parent_parser.add_argument(
        "-v",
        "--verbose",
        action="count",
        default=0,
        dest="verbosity",
        help="increase the level of verbosity",
    )

    subparsers = parser.add_subparsers(dest="subcommand", required=True)

    subparsers.add_parser("out", help="List venvs", parents=[parent_parser])
    subparsers.add_parser("list", help="List venvs", parents=[parent_parser])
    subparsers.add_parser(
        "init", help="Initialisation instructions for pvenv", parents=[parent_parser]
    )
    subparsers.add_parser(
        "deactivate", help="Deactivate a venv", parents=[parent_parser]
    )

    invenv_parser = subparsers.add_parser(
        "in", help="Export new variables for the venv", parents=[parent_parser]
    )
    invenv_parser.add_argument("env_vars", nargs="*")
    invenv_parser.add_argument("-f", "--files", action="append", default=[])

    avenv_parser = subparsers.add_parser(
        "activate", help="Activate a venv", parents=[parent_parser]
    )
    avenv_parser.add_argument("venv")
    avenv_parser.add_argument("-n", "--no-cd", dest="cd", action="store_false")

    mkvenv_parser = subparsers.add_parser(
        "make", help="Make a new venv", parents=[parent_parser]
    )
    mkvenv_parser.add_argument("venv", nargs="?", default=Path().absolute().name)
    mkvenv_parser.add_argument("-e", "--environments", action="append", default=[])
    mkvenv_parser.add_argument("-P", "--project", default=Path().absolute().as_posix())
    mkvenv_parser.add_argument("-p", "--python", default="current")

    rmvenv_parser = subparsers.add_parser(
        "rm", help="Remove virtualenvs", parents=[parent_parser]
    )
    rmvenv_parser.add_argument("venvs_to_remove", nargs="*")

    args = parser.parse_args()
    if args.verbosity > 0:
        sys.tracebacklimit = 1000

    return args



from pvenv.subcommands import (
    avenv,
    dvenv,
    initvenv,
    invenv,
    lsvenv,
    mkvenv,
    outvenv,
    rmvenv,
)

__all__ = [
    "avenv",
    "dvenv",
    "initvenv",
    "invenv",
    "lsvenv",
    "mkvenv",
    "outvenv",
    "rmvenv",
]



from __future__ import annotations

import shutil
from typing import TYPE_CHECKING

from pvenv.subcommands.base import BaseCommand

if TYPE_CHECKING:
    from argparse import Namespace
    from pathlib import Path


class Command(BaseCommand):
    def __init__(self, options: Namespace) -> None:
        super().__init__(options)
        self.base_dir: Path = options.base_dir
        self.venv_to_remove: list[str] = options.venvs_to_remove

    def run(self) -> None:
        for venv in self.venv_to_remove:
            venv_path = self.base_dir.joinpath(venv)
            if not venv_path.exists():
                print(f"{venv} does not exist, skipping...")
                continue
            try:
                shutil.rmtree(venv_path)
            except (PermissionError, NotADirectoryError):
                print(f"Cannot delete {venv}, skipping...")
                continue

            print(f"{venv} removed successfully!")



from __future__ import annotations

import os
from typing import TYPE_CHECKING

from pvenv.subcommands.base import BaseCommand

if TYPE_CHECKING:
    from argparse import Namespace


class Command(BaseCommand):
    def __init__(self, options: Namespace) -> None:
        super().__init__(options)
        self.env_vars: list[str] = options.env_vars
        self.files = options.files

    @staticmethod
    def parse_env_var(line: str) -> list[str]:
        return line.split("=", maxsplit=1)

    def run(self) -> None:
        new_vars = {}
        for file in self.files:
            with file.open() as f:
                for raw_line in f:
                    line = raw_line.strip()
                    if line and not line.startswith("#"):
                        key, value = self.parse_env_var(line)
                        new_vars[key] = value
        for env_var in self.env_vars:
            key, value = self.parse_env_var(env_var)
            new_vars[key] = value

        if new_vars:
            if self._prefix in os.environ:
                msg = "Already in a venv, aborting..."
                raise RuntimeError(msg)
            self.print(f"export {self._prefix}=true")
        for key, value in new_vars.items():
            if key in os.environ:
                self.print(f"export {self._prefix}_{key}={os.getenv(key, '')}")
            else:
                self.print(f"export {self._prefix}_unset_{key}=")
            self.print(f"export {key}={value}")



from __future__ import annotations

import os
from pathlib import Path
from typing import TYPE_CHECKING

from pvenv.subcommands.base import BaseCommand

if TYPE_CHECKING:
    from argparse import Namespace

DEV_NULL = Path(os.devnull)


class Command(BaseCommand):
    def __init__(self, options: Namespace) -> None:
        super().__init__(options)
        self.base_dir: Path = options.base_dir
        self.venv: str = options.venv
        self.python: str = options.python
        self.project: Path = Path(options.project).absolute()
        self.environments: list[Path] = self._get_environments()

    def _get_project(self) -> Path:
        project = self._options.project
        return Path(project).absolute() if project else DEV_NULL

    def _get_environments(self) -> list[Path]:
        environments = self._options.environments
        if self.project == DEV_NULL and environments:
            msg = "Cannot set environ in independent"
            raise RuntimeError(msg)
        return [self.project.joinpath(environ) for environ in environments]

    def run(self) -> None:
        venv_path = self.base_dir.joinpath(self.venv)
        if venv_path.exists():
            msg = f"Venv {self.venv} already exists, aborting..."
            raise RuntimeError(msg)

        if self.python == "current":
            self.print(f"python -m venv {venv_path}")
        elif self.python:
            self.print(f"pyenv shell {self.python}")
            self.print(f"python -m venv {venv_path}")
        else:
            self.print(f"mkdir -p {venv_path}")

        if self.project != DEV_NULL:
            self.print(f"echo {self.project} > {venv_path}/.project")

        for i, environment in enumerate(self.environments):
            if i == 0:
                self.print(f": > {venv_path}/.environment")
            self.print(f"echo {environment} >> {venv_path}/.environment")

        self.print(f"avenv {self.venv}")

        if self.python:
            self.print("pip install --upgrade pip")



import os
from argparse import Namespace
from typing import Any

PREFIX = chr(2)


class BaseCommand:
    def __init__(self, options: Namespace) -> None:
        self._prefix = "_pvenv_env"
        self._options = options

    def run(self) -> None:
        msg = f"{self.__class__.__name__} must implement run"
        raise NotImplementedError(msg)

    @staticmethod
    def print(*args: str, **kwargs: Any) -> None:
        if not os.getenv("PVENV_DEBUG"):
            args = tuple(f"{PREFIX}{arg}" for arg in args)
        print(*args, **kwargs)



from argparse import Namespace
from typing import TYPE_CHECKING

from pvenv.subcommands.base import BaseCommand

if TYPE_CHECKING:
    from pathlib import Path


class Command(BaseCommand):
    def __init__(self, options: Namespace) -> None:
        super().__init__(options)
        self.base_dir: Path = options.base_dir

    def run(self) -> None:
        for directory in sorted(self.base_dir.glob("*")):
            if directory.is_dir():
                print(directory.name)



from argparse import Namespace
from pathlib import Path

from pvenv.subcommands.base import BaseCommand


class Command(BaseCommand):
    def __init__(self, options: Namespace) -> None:
        super().__init__(options)

    def run(self) -> None:
        print(
            "Add the following line to your shell rc:\n"
            f". {Path(__file__).parent.joinpath('scripts/pvenv.sh').absolute()}"
        )



from typing import TYPE_CHECKING

from pvenv import subcommands
from pvenv.lib.parser import parse_args

if TYPE_CHECKING:
    from types import ModuleType


def main() -> None:
    args = parse_args()
    module: ModuleType
    if args.subcommand == "in":
        module = subcommands.invenv
    elif args.subcommand == "init":
        module = subcommands.initvenv
    elif args.subcommand == "out":
        module = subcommands.outvenv
    elif args.subcommand == "activate":
        module = subcommands.avenv
    elif args.subcommand == "deactivate":
        module = subcommands.dvenv
    elif args.subcommand == "list":
        module = subcommands.lsvenv
    elif args.subcommand == "make":
        module = subcommands.mkvenv
    elif args.subcommand == "rm":
        module = subcommands.rmvenv
    module.Command(args).run()
